@@ -1,4 +1,4 @@
-# Gufw 14.04.2 - http://gufw.org
+# Gufw 14.10.1 - http://gufw.org
 # Copyright (C) 2008-2014 Marcos Alvarez Costales https://launchpad.net/~costales
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@ class Firewall():
         self.status = self.backend.get_status()
         self.incoming = self.backend.get_policy('incoming')
         self.outgoing = self.backend.get_policy('outgoing')
+        self.routed   = self.backend.get_policy('routed')
         self.ufw_logging = self.backend.get_ufw_logging()
         
         self.gufw_logging = True # By default
@@ -67,7 +68,7 @@ class Firewall():
         self.set_cfg_value('Profile', profile)
         
         # Get new profile
-        (new_status, new_incoming, new_outgoing, new_rules) = self.backend.get_profile_values(profile)
+        (new_status, new_incoming, new_outgoing, new_routed, new_rules) = self.backend.get_profile_values(profile)
         
         # Adding Gufw rules in new profile
         for new_row in new_rules:
@@ -75,20 +76,25 @@ class Firewall():
                 result = self.backend.add_rule('', # Insert in order
                                                 new_row['policy'],
                                                 new_row['direction'],
-                                                new_row['iface'], 
-                                                new_row['logging'], 
-                                                new_row['protocol'], 
-                                                new_row['from_ip'], 
-                                                new_row['from_port'], 
-                                                new_row['to_ip'], 
+                                                new_row['iface'],
+                                                new_row['routed'],
+                                                new_row['logging'],
+                                                new_row['protocol'],
+                                                new_row['from_ip'],
+                                                new_row['from_port'],
+                                                new_row['to_ip'],
                                                 new_row['to_port'])
                 line = _("Appending new rules: ") + result[0] + ' > ' + result[1].replace('\n', ' | ')
                 operation.append(line)
         
-        # New status, incoming and outgoing
-        self.status = new_status
+        # New status, incoming, outgoing, routed
+        self.status   = new_status
         self.incoming = new_incoming
         self.outgoing = new_outgoing
+        if new_routed == 'disabled' and self.backend.get_policy('routed') != 'disabled':
+            self.routed = self.backend.get_policy('routed')
+        else:
+            self.routed = new_routed
         
         return operation
     
@@ -97,7 +103,7 @@ class Firewall():
     
     def add_profile(self, profile):
         self.all_profiles.append(profile)
-        self.backend.set_profile_values(profile, self.status, self.incoming, self.outgoing, []) # Will create profile
+        self.backend.set_profile_values(profile, self.status, self.incoming, self.outgoing, self.routed, []) # Will create profile
     
     def delete_profile(self, profile):
         self.all_profiles.remove(profile)
@@ -112,9 +118,9 @@ class Firewall():
     def import_profile(self, profile_file):
         new_profile = os.path.basename(profile_file)
         new_profile = os.path.splitext(new_profile)[0]
-        (status, incoming, outgoing, rules) = self.backend.get_profile_values(profile_file)
+        (status, incoming, outgoing, routed, rules) = self.backend.get_profile_values(profile_file)
         self.add_profile(new_profile)
-        self.backend.set_profile_values(new_profile, status, incoming, outgoing, rules)
+        self.backend.set_profile_values(new_profile, status, incoming, outgoing, routed, rules)
         
     def export_profile(self, profile_file):
         self.backend.export_profile(self.profile, profile_file)
@@ -127,26 +133,34 @@ class Firewall():
     def set_status(self, status):
         self.status = status
         self.backend.set_status(status)
-        self.backend.set_profile_values(self.profile, status, self.incoming, self.outgoing, self.get_rules())
+        self.backend.set_profile_values(self.profile, status, self.incoming, self.outgoing, self.routed, self.get_rules())
     
     def get_policy(self, policy):
         if policy == 'incoming':
             return self.incoming
         elif policy == 'outgoing':
             return self.outgoing
+        elif policy == 'routed':
+            if self.backend.get_policy('routed') != 'disabled':
+                return self.routed
+            else:
+                return self.backend.get_policy('routed')
     
     def set_policy(self, policy, value):
         if policy == 'incoming':
             self.incoming = value
             self.backend.set_policy(policy, value)
-        else:
+        elif policy == 'outgoing':
             self.outgoing = value
             self.backend.set_policy(policy, value)
+        elif policy == 'routed':
+            self.routed = value
+            self.backend.set_policy(policy, value)
         
-        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.get_rules())
+        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.routed, self.get_rules())
     
     def reset(self):
-        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, [])
+        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.routed, [])
         self.backend.reset_fw()
     
     # RULES
@@ -160,16 +174,19 @@ class Firewall():
         profile_rules = self._get_rules_profile()
         return self._compose_rules(ufw_rules, profile_rules)
     
-    def add_rule(self, description, insert, policy, direction, iface, logging, proto, from_ip, from_port, to_ip, to_port):
+    def get_number_rules(self):
+        return self.backend.get_number_rules()
+    
+    def add_rule(self, description, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port):
         rules_before = self.get_rules()
         rules_profile_before = self._get_rules_profile()
         
-        cmd = self.backend.add_rule(insert, policy, direction, iface, logging, proto, from_ip, from_port, to_ip, to_port)
+        cmd = self.backend.add_rule(insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
         rules_after = self.get_rules()
         
         if len(rules_before) != len(rules_after):
             cmd.insert(0, True)
-            self._regenerate_file_profile(rules_before, rules_profile_before, rules_after, description, cmd[1], policy, direction, iface, logging, proto, from_ip, from_port, to_ip, to_port)
+            self._regenerate_file_profile(rules_before, rules_profile_before, rules_after, description, cmd[1], policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
         else:
             cmd.insert(0, False)
         return cmd # For logging
@@ -240,8 +257,8 @@ class Firewall():
     
     def get_net_interfaces(self):
         return interfaces()
-	
-	
+    
+    
     # STUFF
     def _read_default_profile(self):
         default_profile = self.get_cfg_value('Profile')
@@ -250,7 +267,7 @@ class Firewall():
             default_profile = _("Home")
         # Error in config file? > Regenerate
         if default_profile and (not default_profile in self.all_profiles):
-            self.backend.set_profile_values(default_profile, self.status, self.incoming, self.outgoing, [])
+            self.backend.set_profile_values(default_profile, self.status, self.incoming, self.outgoing, self.routed, [])
             self.all_profiles.append(default_profile)
 
         return default_profile
@@ -265,9 +282,9 @@ class Firewall():
                 profiles.append(os.path.splitext(os.path.basename(profile))[0].replace(' ', '_')) # Filename without path and extension
             
             if not profiles: # First run
-                self.backend.set_profile_values(_("Public").replace(' ', '_'), True, 'reject',  'allow', [])
-                self.backend.set_profile_values(_("Office").replace(' ', '_'), True, 'deny',    'allow', [])
-                self.backend.set_profile_values(_("Home").replace(' ', '_'), self.status, self.incoming, self.outgoing, [])
+                self.backend.set_profile_values(_("Public").replace(' ', '_'), True, 'reject',  'allow', 'allow', [])
+                self.backend.set_profile_values(_("Office").replace(' ', '_'), True, 'deny',    'allow', 'allow', [])
+                self.backend.set_profile_values(_("Home").replace(' ', '_'), self.status, self.incoming, self.outgoing, self.routed, [])
                 self.set_cfg_value('Profile', _("Home").replace(' ', '_'))
         
         return profiles
@@ -313,6 +330,7 @@ class Firewall():
                     'to_ip'      : '', # to_ip
                     'to_port'    : '', # to_port
                     'iface'      : '', # iface
+                    'routed'     : '', # routed
                     'logging'    : ''} # logging
             
             for profile_rule in profile_rules:
@@ -328,6 +346,7 @@ class Firewall():
                             'to_ip'       : profile_rule['to_ip'],       # to_ip
                             'to_port'     : profile_rule['to_port'],     # to_port
                             'iface'       : profile_rule['iface'],       # iface
+                            'routed'      : profile_rule['routed'],      # routed
                             'logging'     : profile_rule['logging']}     # logging
                     break
             
@@ -337,25 +356,26 @@ class Firewall():
     
     def _get_rules_profile(self):
         rules_profile = []
-        (status, incoming, outgoing, rules) = self.backend.get_profile_values(self.profile)
+        (status, incoming, outgoing, routed, rules) = self.backend.get_profile_values(self.profile)
         for rule in rules:
-            conver_rules = {'ufw_rule'    : rule['ufw_rule'],                              
-                            'description' : rule['description'],         
-                            'command'     : rule['command'],               
-                            'policy'      : rule['policy'],                
-                            'direction'   : rule['direction'],             
-                            'protocol'    : rule['protocol'],              
-                            'from_ip'     : rule['from_ip'],               
-                            'from_port'   : rule['from_port'],             
-                            'to_ip'       : rule['to_ip'],                 
-                            'to_port'     : rule['to_port'],               
-                            'iface'       : rule['iface'],                 
-                            'logging'     : rule['logging']}               
+            conver_rules = {'ufw_rule'    : rule['ufw_rule'],
+                            'description' : rule['description'],
+                            'command'     : rule['command'],
+                            'policy'      : rule['policy'],
+                            'direction'   : rule['direction'],
+                            'protocol'    : rule['protocol'],
+                            'from_ip'     : rule['from_ip'],
+                            'from_port'   : rule['from_port'],
+                            'to_ip'       : rule['to_ip'],
+                            'to_port'     : rule['to_port'],
+                            'iface'       : rule['iface'],
+                            'routed'      : rule['routed'],
+                            'logging'     : rule['logging']}
             rules_profile.append(conver_rules)
         
         return rules_profile
         
-    def _regenerate_file_profile(self, ufw_before, profile_before, ufw_after, description='', cmd='', policy='', direction='', iface='', logging='', proto='', from_ip='', from_port='', to_ip='', to_port=''):
+    def _regenerate_file_profile(self, ufw_before, profile_before, ufw_after, description='', cmd='', policy='', direction='', iface='', routed='', logging='', proto='', from_ip='', from_port='', to_ip='', to_port=''):
         """Here there are dragons!"""
         final_rules = []            
         # Operation ufw_before  ufw_after   profile_before  profile_after
@@ -368,35 +388,37 @@ class Firewall():
             for profile_rule in profile_before:
                 if ufw_rule['ufw_rule'] == profile_rule['ufw_rule']: # Complete from previous profile
                     found = True
-                    new_rule = {'ufw_rule'    : profile_rule['ufw_rule'],    
-                                'description' : profile_rule['description'], 
-                                'command'     : profile_rule['command'],     
-                                'policy'      : profile_rule['policy'],      
-                                'direction'   : profile_rule['direction'],   
-                                'protocol'    : profile_rule['protocol'],    
-                                'from_ip'     : profile_rule['from_ip'],     
-                                'from_port'   : profile_rule['from_port'],   
-                                'to_ip'       : profile_rule['to_ip'],       
-                                'to_port'     : profile_rule['to_port'],     
-                                'iface'       : profile_rule['iface'],       
-                                'logging'     : profile_rule['logging']}     
+                    new_rule = {'ufw_rule'    : profile_rule['ufw_rule'],
+                                'description' : profile_rule['description'],
+                                'command'     : profile_rule['command'],
+                                'policy'      : profile_rule['policy'],
+                                'direction'   : profile_rule['direction'],
+                                'protocol'    : profile_rule['protocol'],
+                                'from_ip'     : profile_rule['from_ip'],
+                                'from_port'   : profile_rule['from_port'],
+                                'to_ip'       : profile_rule['to_ip'],
+                                'to_port'     : profile_rule['to_port'],
+                                'iface'       : profile_rule['iface'],
+                                'routed'      : profile_rule['routed'],
+                                'logging'     : profile_rule['logging']}
             if not found:
                 if ufw_before.count(ufw_rule) == 0: # New: Complete from parameters
                     new_rule = {'ufw_rule'    : ufw_rule['ufw_rule'],
-                                'description' : description, 
+                                'description' : description,
                                 'command'     : cmd,
-                                'policy'      : policy,    
-                                'direction'   : direction, 
-                                'protocol'    : proto,    
-                                'from_ip'     : from_ip,   
-                                'from_port'   : from_port, 
-                                'to_ip'       : to_ip,   
-                                'to_port'     : to_port,   
-                                'iface'       : iface,   
-                                'logging'     : logging}   
+                                'policy'      : policy,
+                                'direction'   : direction,
+                                'protocol'    : proto,
+                                'from_ip'     : from_ip,
+                                'from_port'   : from_port,
+                                'to_ip'       : to_ip,
+                                'to_port'     : to_port,
+                                'iface'       : iface,
+                                'routed'      : routed,
+                                'logging'     : logging}
                 else: # Old ufw rule: Nothing
                     continue
                     
             final_rules.append(new_rule) # Just adding rules, a regenerate will be update the rules
         
-        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, final_rules)
+        self.backend.set_profile_values(self.profile, self.status, self.incoming, self.outgoing, self.routed, final_rules)
