@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Gufw 14.04.2 - http://gufw.org
+# Gufw 14.10.1 - http://gufw.org
 # Copyright (C) 2008-2014 Marcos Alvarez Costales https://launchpad.net/~costales
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -66,17 +66,19 @@ class Add:
         self.simple_protocol  = self.builder.get_object('simple_protocol')
         self.simple_port      = self.builder.get_object('simple_port')
         
-        self.advanced_rule_name = self.builder.get_object('advanced_rule_name')
-        self.advanced_insert    = self.builder.get_object('advanced_insert')
-        self.advanced_policy    = self.builder.get_object('advanced_policy')
-        self.advanced_direction = self.builder.get_object('advanced_direction')
-        self.advanced_iface     = self.builder.get_object('advanced_iface')
-        self.advanced_log       = self.builder.get_object('advanced_log')
-        self.advanced_protocol  = self.builder.get_object('advanced_protocol')
-        self.advanced_from_ip   = self.builder.get_object('advanced_from_ip')
-        self.advanced_from_port = self.builder.get_object('advanced_from_port')
-        self.advanced_to_ip     = self.builder.get_object('advanced_to_ip')
-        self.advanced_to_port   = self.builder.get_object('advanced_to_port')
+        self.advanced_rule_name  = self.builder.get_object('advanced_rule_name')
+        self.advanced_insert     = self.builder.get_object('advanced_insert')
+        self.advanced_policy     = self.builder.get_object('advanced_policy')
+        self.advanced_direction  = self.builder.get_object('advanced_direction')
+        self.advanced_iface      = self.builder.get_object('advanced_iface')
+        self.advanced_routed_img = self.builder.get_object('routed_img')
+        self.advanced_routed     = self.builder.get_object('advanced_routed')
+        self.advanced_log        = self.builder.get_object('advanced_log')
+        self.advanced_protocol   = self.builder.get_object('advanced_protocol')
+        self.advanced_from_ip    = self.builder.get_object('advanced_from_ip')
+        self.advanced_from_port  = self.builder.get_object('advanced_from_port')
+        self.advanced_to_ip      = self.builder.get_object('advanced_to_ip')
+        self.advanced_to_port    = self.builder.get_object('advanced_to_port')
 
     def _set_initial_values(self):
         self._set_app_combobox()
@@ -92,16 +94,24 @@ class Add:
         self.advanced_policy.set_active(0)
         self.advanced_direction.set_active(0)
         self.advanced_iface.append_text(_("All Interfaces"))
+        self.advanced_routed.append_text(_("Not Forward"))
         for ifaceName in self.gufw.fw.get_net_interfaces():
             self.advanced_iface.append_text(ifaceName)
+            self.advanced_routed.append_text(ifaceName)
         self.advanced_iface.set_active(0)
+        self.advanced_routed.set_active(0)
         self.advanced_log.set_active(0)
         self.advanced_protocol.set_active(0)
+        
+        if self.gufw.fw.get_policy('routed') == 'disabled':
+            self.advanced_routed_img.set_visible(False)
+            self.advanced_routed.set_visible(False)
     
     def show_win(self):
         if self.app_filter.get_text():
             self.app_filter.set_text('')
         self.win_add.show()
+        
     
     def set_add_from_report(self, protocol='', port='', address='', name=''):
         self.advanced_rule_name.set_text(name)
@@ -191,39 +201,15 @@ class Add:
     
     def _set_from_port_sensitive(self, value=True):
         self.advanced_protocol.set_sensitive(value)
-        self.advanced_iface.set_sensitive(value)
         self.advanced_from_ip.set_sensitive(value)
         self.advanced_to_ip.set_sensitive(value)
         self.advanced_to_port.set_sensitive(value)
-        if value:
-            if self.advanced_from_ip.get_text() or self.advanced_to_ip.get_text():
-                self.advanced_iface.set_sensitive(True)
-            else:
-                self.advanced_iface.set_sensitive(False)
     
     def _set_to_port_sensitive(self, value=True):
         self.advanced_protocol.set_sensitive(value)
-        self.advanced_iface.set_sensitive(value)
         self.advanced_from_ip.set_sensitive(value)
         self.advanced_to_ip.set_sensitive(value)
         self.advanced_from_port.set_sensitive(value)
-        if value:
-            if self.advanced_from_ip.get_text() or self.advanced_to_ip.get_text():
-                self.advanced_iface.set_sensitive(True)
-            else:
-                self.advanced_iface.set_sensitive(False)
-    
-    def on_advanced_from_ip_changed(self, widget, data=None):
-        if self.advanced_from_ip.get_text() or self.advanced_to_ip.get_text():
-            self.advanced_iface.set_sensitive(True)
-        else:
-            self.advanced_iface.set_sensitive(False)
-    
-    def on_advanced_to_ip_changed(self, widget, data=None):
-        if self.advanced_from_ip.get_text() or self.advanced_to_ip.get_text():
-            self.advanced_iface.set_sensitive(True)
-        else:
-            self.advanced_iface.set_sensitive(False)
     
     def on_preconfig_category_changed(self, widget, data=None):
         self.updating_subcategories = True
@@ -317,7 +303,15 @@ class Add:
         self.advanced_from_port.set_text('')
     def on_advanced_to_port_icon_press(self, widget, data=None, data2=None):
         self.advanced_to_port.set_text('')
-         
+    
+    def on_advanced_iface_changed(self, widget, data=None, data2=None):
+        if self.advanced_iface.get_active_text() != _("All Interfaces"):
+            self.advanced_routed.set_sensitive(True)
+            self.advanced_routed.set_tooltip_text(_("The IP/Port will be forward to this interface"))
+        else:
+            self.advanced_routed.set_sensitive(False)
+            self.advanced_routed.set_tooltip_text(_("You need to set an Interface for forwarding to this another interface"))
+    
     def on_preconfig_app_changed(self, widget, data=None):
         if self.preconfig_app.get_active_text() != None:
             tooltip = self.apps.get_app(self.preconfig_app.get_active_text())[0] + '\n' + _("Ports: ") + self.apps.get_app(self.preconfig_app.get_active_text())[1]
@@ -340,26 +334,20 @@ class Add:
         elif self.tabs.get_current_page() == 2:
             self._add_rule_advanced()
     
-    def _add(self, profile='', name='', policy='', direction='', proto='', from_ip='', from_port='', to_ip='', to_port='', insert='', iface='', logging=''):
+    def _add(self, profile='', name='', policy='', direction='', proto='', from_ip='', from_port='', to_ip='', to_port='', insert='', iface='', routed='', logging=''):
         flag_OK = True
         flag_Warning = False
         
         # Split possible ports > 135,139,445/tcp|137,138/udp
         for from_split in from_port.split('|'):
             for to_split in to_port.split('|'):
-                if ('/tcp' in from_split or 
-                    '/udp' in from_split or
-                    '/tcp' in to_split   or
-                    '/udp' in to_split):
-                    proto = ''
-                
                 from_port = from_split
                 to_port = to_split
                 if direction == 'both':
-                    cmd = self.gufw.fw.add_rule(name, insert, policy, 'in',  iface, logging, proto, from_ip, from_port, to_ip, to_port)
-                    cmd = self.gufw.fw.add_rule(name, insert, policy, 'out', iface, logging, proto, from_ip, from_port, to_ip, to_port)
+                    cmd = self.gufw.fw.add_rule(name, insert, policy, 'in',  iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
+                    cmd = self.gufw.fw.add_rule(name, insert, policy, 'out', iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
                 else:
-                    cmd = self.gufw.fw.add_rule(name, insert, policy, direction, iface, logging, proto, from_ip, from_port, to_ip, to_port)
+                    cmd = self.gufw.fw.add_rule(name, insert, policy, direction, iface, routed, logging, proto, from_ip, from_port, to_ip, to_port)
                 
                 if cmd[0]:
                     flag_Warning = True
@@ -380,13 +368,13 @@ class Add:
     
     def _add_rule_preconfig(self):
         self._add(self.gufw.fw.get_profile(),                                    # profile
-                 self.preconfig_app.get_active_text(),                            # name
+                 self.preconfig_app.get_active_text(),                           # name
                  self.gufw.NUM2POLICY[self.preconfig_policy.get_active()],       # policy
                  self.gufw.NUM2DIRECTION[self.preconfig_direction.get_active()], # direction
-                 '',                                                               # proto
-                 '',                                                               # from IP
-                 '',                                                               # from port
-                 '',                                                               # to IP
+                 '',                                                             # proto
+                 '',                                                             # from IP
+                 '',                                                             # from port
+                 '',                                                             # to IP
                  self.apps.get_app(self.preconfig_app.get_active_text())[1])     # to port[/proto][|port[/proto]]
         self.gufw.print_rules(self.gufw.fw.get_rules())
     
@@ -400,19 +388,19 @@ class Add:
             return
         
         self._add(self.gufw.fw.get_profile(),                                 # profile
-                 self.simple_rule_name.get_text(),                             # name
+                 self.simple_rule_name.get_text(),                            # name
                  self.gufw.NUM2POLICY[self.simple_policy.get_active()],       # policy
                  self.gufw.NUM2DIRECTION[self.simple_direction.get_active()], # direction
                  self.gufw.NUM2PROTO[self.simple_protocol.get_active()],      # protocol
-                 '',                                                            # from IP
-                 '',                                                            # from port
-                 '',                                                            # to IP
-                 self.simple_port.get_text())                                  # to port
+                 '',                                                          # from IP
+                 '',                                                          # from port
+                 '',                                                          # to IP
+                 self.simple_port.get_text())                                 # to port
         self.gufw.print_rules(self.gufw.fw.get_rules())
     
     def _add_rule_advanced(self):
         # Validations
-        if not self.gufw.validate_rule(self.win_add, self.advanced_from_ip.get_text(), self.advanced_from_port.get_text(), self.advanced_to_ip.get_text(), self.advanced_to_port.get_text()):
+        if not self.gufw.validate_rule(self.win_add, self.advanced_from_ip.get_text(), self.advanced_from_port.get_text(), self.advanced_to_ip.get_text(), self.advanced_to_port.get_text(), self.advanced_insert.get_text(), self.advanced_routed.get_active_text()):
             return
         
         insert = self.advanced_insert.get_text()
@@ -423,27 +411,28 @@ class Add:
         if self.advanced_iface.get_sensitive() and self.advanced_iface.get_active_text() != _("All Interfaces"):
             iface = self.advanced_iface.get_active_text()
         
+        routed = ''
+        if self.advanced_routed.get_sensitive() and self.advanced_routed.get_active_text() != _("Not Forward"):
+            routed = self.advanced_routed.get_active_text()
+        
         to_ip = to_port = from_ip = from_port = ''
-        if self.advanced_from_ip.get_sensitive():
-            from_ip = self.advanced_from_ip.get_text()
-        if self.advanced_from_port.get_sensitive():
-            from_port = self.advanced_from_port.get_text()
-        if self.advanced_to_ip.get_sensitive():
-            to_ip = self.advanced_to_ip.get_text()
-        if self.advanced_to_port.get_sensitive():
-            to_port = self.advanced_to_port.get_text()
+        from_ip = self.advanced_from_ip.get_text()
+        from_port = self.advanced_from_port.get_text()
+        to_ip = self.advanced_to_ip.get_text()
+        to_port = self.advanced_to_port.get_text()
         
         self._add(self.gufw.fw.get_profile(),                                   # profile
-                 self.advanced_rule_name.get_text(),                             # name
+                 self.advanced_rule_name.get_text(),                            # name
                  self.gufw.NUM2POLICY[self.advanced_policy.get_active()],       # policy
                  self.gufw.NUM2DIRECTION[self.advanced_direction.get_active()], # direction
                  self.gufw.NUM2PROTO[self.advanced_protocol.get_active()],      # protocol
-                 from_ip,                                                         # from IP
-                 from_port,                                                       # from port
-                 to_ip,                                                           # to IP
-                 to_port,                                                         # to port
-                 insert,                                                          # insert number
-                 iface,                                                           # interface
+                 from_ip,                                                       # from IP
+                 from_port,                                                     # from port
+                 to_ip,                                                         # to IP
+                 to_port,                                                       # to port
+                 insert,                                                        # insert number
+                 iface,                                                         # interface
+                 routed,                                                        # routed
                  self.gufw.NUM2LOGGING[self.advanced_log.get_active()])         # logging        
         self.gufw.print_rules(self.gufw.fw.get_rules())
 
