@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Gufw 14.04.2 - http://gufw.org
+# Gufw 14.10.1 - http://gufw.org
 # Copyright (C) 2008-2014 Marcos Alvarez Costales https://launchpad.net/~costales
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -43,7 +43,8 @@ class Gufw:
     POLICY2NUM = {'allow': 0,
                   'deny': 1,
                   'reject': 2,
-                  'limit': 3 }
+                  'limit': 3,
+                  'disabled': 4 }
     POLICY2COLOR = {'allow': RED,
                     'deny': GREEN,
                     'reject': BLUE,
@@ -73,7 +74,6 @@ class Gufw:
         self.builder.add_from_file('/usr/share/gufw/ui/gufw.ui')
         
         self._set_objects_name()
-        self.listening = ListeningReport(self)
         self._set_initial_values()
         self.winadd = Add(self)
         
@@ -88,22 +88,22 @@ class Gufw:
         self.switchStatus = self.builder.get_object('switchStatus')
         self.incoming     = self.builder.get_object('incoming')
         self.outgoing     = self.builder.get_object('outgoing')
+        self.routed_lbl   = self.builder.get_object('routed_lbl')
+        self.box_routed   = self.builder.get_object('box_routed')
+        self.routed       = self.builder.get_object('routed')
         self.shield       = self.builder.get_object('shield')
         
-        self.rules_expander  = self.builder.get_object('expanderRules')
         self.rules_box       = self.builder.get_object('boxRules')
         self.rules           = self.builder.get_object('Rules')
         self.show_add_btn    = self.builder.get_object('btnAddRule')
         self.detele_rule_btn = self.builder.get_object('btnDeleteRule')
         self.edit_rule_btn   = self.builder.get_object('btnEditRule')
         
-        self.report_expander = self.builder.get_object('expanderReport')
         self.report_box      = self.builder.get_object('boxReport')
         self.report          = self.builder.get_object('Report')
         self.report_rule_btn = self.builder.get_object('btnReportRule')
         self.report_waiting  = self.builder.get_object('report_waiting')
         
-        self.log_expander = self.builder.get_object('expanderLog')
         self.log_box      = self.builder.get_object('boxLog')
         self.log          = self.builder.get_object('log')
         self.log_txt      = self.log.get_buffer()
@@ -111,8 +111,15 @@ class Gufw:
         self.log_delete   = self.builder.get_object('btnLogCopy')
         
         self.web = self.builder.get_object('boxWeb')
-        self.web_context = WebKit.WebView()
-        self.web.add(self.web_context)
+        self.web_content = WebKit.WebView()
+        self.web.add(self.web_content)
+        # For ORCA
+        self.tuto_label = Gtk.Label()
+        self.tuto_label.set_text(_("Getting started"))
+        self.tuto_label.set_mnemonic_widget(self.web)
+        
+        settings = self.web_content.get_settings()
+        settings.set_property('enable-caret-browsing', True)
         
         self.statusbar = self.builder.get_object('statusmsg')
         self.progress  = self.builder.get_object('progress')
@@ -129,15 +136,16 @@ class Gufw:
                                           str,  # 8  to_ip
                                           str,  # 9  to_port
                                           str,  # 10 iface
-                                          str,  # 11 logging
-                                          str,  # 12 color
-                                          int)  # 13 number (for deleting and updating)
+                                          str,  # 11 routed
+                                          str,  # 12 logging
+                                          str,  # 13 color
+                                          int)  # 14 number (for deleting and updating)
         self.tv_rules = self.rules
         self.tv_rules.set_model(self.rules_model)
         self.tv_rules.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
         
         # Listening report
-        self.listening_model = Gtk.ListStore(str, # 0 protocol
+        self.listening_model = Gtk.ListStore(str, # 0  protocol
                                               str, # 1 port
                                               str, # 2 address
                                               str, # 3 app
@@ -155,6 +163,46 @@ class Gufw:
         self.txt_tag_blue   = self.log_txt.create_tag('colored_blue',   foreground=self.BLUE)
         self.txt_tag_gray   = self.log_txt.create_tag('colored_gray',   foreground=self.GRAY)
         self.txt_tag_black  = self.log_txt.create_tag('colored_black',  foreground=self.BLACK)
+        
+        # Stack init
+        stack = Gtk.Stack()
+        stack.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT_RIGHT)
+        stack.set_transition_duration(700)
+        # Stack 1 = Tutorial
+        vbox_home = Gtk.Box()
+        self.viewport_home  = self.builder.get_object('viewport_home')
+        self.viewport_home.reparent(vbox_home)
+        stack.add_titled(vbox_home, "tutorial", _("Getting started"))
+        stack.child_set_property(vbox_home, "icon-name", "gtk-home") # Set icon
+        # Stack 2 = Rules
+        vbox_rules = Gtk.Box()
+        self.viewport_rules  = self.builder.get_object('viewport_rules')
+        self.viewport_rules.reparent(vbox_rules)
+        stack.add_titled(vbox_rules, "rules", _("Rules"))
+        # Stack 3 = Report
+        vbox_report = Gtk.Box()
+        self.viewport_report  = self.builder.get_object('viewport_report')
+        self.viewport_report.reparent(vbox_report)
+        stack.add_titled(vbox_report, "report", _("Report"))
+        # Stack 4 = Log
+        vbox_log = Gtk.Box()
+        self.viewport_log  = self.builder.get_object('viewport_log')
+        self.viewport_log.reparent(vbox_log)
+        stack.add_titled(vbox_log, "log", _("Log"))
+        # Stack Compose
+        stack_switcher = Gtk.StackSwitcher()
+        stack_switcher.set_stack(stack)
+        # Horizontal center in 1st row
+        vbox_1row = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        aspect1 = Gtk.Alignment()
+        aspect2 = Gtk.Alignment()
+        vbox_1row.pack_start(aspect1, True, True, 0)
+        vbox_1row.pack_start(stack_switcher, False, False, 0)
+        vbox_1row.pack_start(aspect2, True, True, 0)
+        # Pack
+        self.stack_vbox = self.builder.get_object('stack_vbox')
+        self.stack_vbox.pack_start(vbox_1row, False, True, 0)
+        self.stack_vbox.pack_end(stack, True, True, 0)
     
     def _set_initial_values(self):
         for profile in self.fw.get_all_profiles():
@@ -165,8 +213,10 @@ class Gufw:
         
         self.incoming.set_active(self.POLICY2NUM[self.fw.get_policy('incoming')])
         self.outgoing.set_active(self.POLICY2NUM[self.fw.get_policy('outgoing')])
+        self.routed.set_active(self.POLICY2NUM[self.fw.get_policy('routed')])
         self.incoming.set_sensitive(self.fw.get_status())
         self.outgoing.set_sensitive(self.fw.get_status())
+        self.routed.set_sensitive(self.fw.get_status())
         self.show_add_btn.set_sensitive(self.fw.get_status())
         self.detele_rule_btn.set_sensitive(self.fw.get_status())
         self.edit_rule_btn.set_sensitive(self.fw.get_status())
@@ -175,17 +225,17 @@ class Gufw:
         render_txt = Gtk.CellRendererText()
         
         # Rules
-        tree_header = Gtk.TreeViewColumn(_("Nº"), render_txt, text=13, foreground=12)
+        tree_header = Gtk.TreeViewColumn(_("Nº"), render_txt, text=14, foreground=13)
         tree_header.set_resizable(True)
-        tree_header.set_sort_column_id(13)
+        tree_header.set_sort_column_id(14)
         tree_header.set_sizing(1)
         self.tv_rules.append_column(tree_header)
-        tree_header = Gtk.TreeViewColumn(_("Rule"), render_txt, text=0, foreground=12)
+        tree_header = Gtk.TreeViewColumn(_("Rule"), render_txt, text=0, foreground=13)
         tree_header.set_resizable(True)
         tree_header.set_sort_column_id(0)
         tree_header.set_sizing(1)
         self.tv_rules.append_column(tree_header)
-        tree_header = Gtk.TreeViewColumn(_("Name"), render_txt, text=1, foreground=12)
+        tree_header = Gtk.TreeViewColumn(_("Name"), render_txt, text=1, foreground=13)
         tree_header.set_resizable(True)
         tree_header.set_sort_column_id(1)
         tree_header.set_sizing(1)
@@ -217,38 +267,27 @@ class Gufw:
         tree_header.set_sizing(1)
         self.tv_report.append_column(tree_header)
         
-        if self.fw.get_config_value('ExpandRules') == 'yes':
-            self.web.hide()
-            self.rules_expander.set_expanded(True)
-            self.rules_box.show()
+        self.listening = ListeningReport(self)
         
-        if self.fw.get_config_value('ExpandListening') == 'yes':
-            self.web.hide()
-            self.report_expander.set_expanded(True)
-            self.report_waiting.set_visible(True)
-            self.report_box.show()
-            self.listening.starting()
+        self.add_to_log(self.fw.get_log(), self.GRAY, False)
         
-        if self.fw.get_config_value('ExpandLog') == 'yes':
-            self.web.hide()
-            self.log_expander.set_expanded(True)
-            self.log_box.show()
-            self.add_to_log(self.fw.get_log(), self.GRAY, False)
-        
-        self.web_context.show()
-        self._show_tutorial(self.rules_expander.get_expanded(), self.report_expander.get_expanded(), self.log_expander.get_expanded())
+        self._load_tutorial()
         
         self._set_shield()
         
         self._restore_window_size(self.winMain)
-        self.winMain.show()
+        self.winMain.show_all()
+        
+        if self.fw.get_policy('routed') == 'disabled':
+            self.routed_lbl.set_visible(False)
+            self.box_routed.set_visible(False)
+        
+        if self.fw.get_config_value('ShowDonationBtn') == 'no':
+            self.donateBox.hide()
         
         if self.fw.get_status():
             self.print_rules(self.fw.get_rules())
         
-        if self.fw.get_config_value('ShowDonationBtn') == 'no':
-            self.donateBox.hide()
-    
     def _load_tutorial(self):
         f = open('/usr/share/gufw/media/tutorial/index.html', 'r')
         html_content = f.read()
@@ -282,41 +321,19 @@ class Gufw:
                             faq7_a2 = "ssh user_in_B@IP_of_B -X",
                             faq7_a3 = "sudo /usr/bin/gufw-pkexec -ssh",
                             faq7_a4=_("Gufw will appear in your client \"A\" and all the operations will be apply in the server \"B\". Please, read the community documentation for a complete information."),
-                            faq8_q=_("I want even more! :)"),
-                            faq8_a=_("Visit the community documentation"))
+                            faq8_q=_("I want even more!"),
+                            faq8_a=_("You'll find more information in the community documentation :)"),
+                            faq9_q=_("How can I forward my IP/Ports?"),
+                            faq9_a1=_("To routing policy and rules, you must setup IP forwarding in the system:"),
+                            faq9_a2=_("Close Gufw"),
+                            faq9_a3=_("Uncomment these 3 lines in the file") + " /etc/ufw/sysctl.conf:",
+                            faq9_a4="&nbsp;&nbsp; #net/ipv4/ip_forward=1",
+                            faq9_a5="&nbsp;&nbsp; #net/ipv6/conf/default/forwarding=1",
+                            faq9_a6="&nbsp;&nbsp; #net/ipv6/conf/all/forwarding=1",
+                            faq9_a7=_("Reload the firewall: ") + "sudo ufw reload",
+                            faq9_a8=_("Open Gufw, you'll see a new \"Routed\" option (with an arrow around the shield) and you can forward any rule from the Update window or Advanced Tab in Add window."))
         html = Template(html_content).safe_substitute(replace_html)
-        self.web_context.load_html_string(html, "file:///")
-    
-    def on_expanderRules_activate(self, widget, data=None):
-        if not self.rules_expander.get_expanded():
-            self.rules_box.show()
-            self.print_rules(self.fw.get_rules())
-            self.fw.set_config_value('ExpandRules', 'yes')
-        else:
-            self.rules_box.hide()
-            self.fw.set_config_value('ExpandRules', 'no')
-        self._show_tutorial(not self.rules_expander.get_expanded(), self.report_expander.get_expanded(), self.log_expander.get_expanded())
-    
-    def on_expanderReport_activate(self, widget, data=None):
-        if not self.report_expander.get_expanded():
-            self.report_box.show()
-            self.fw.set_config_value('ExpandListening', 'yes')
-            self.report_waiting.set_visible(True)
-            self.listening.starting()
-        else:
-            self.listening.stopping()
-            self.fw.set_config_value('ExpandListening', 'no')
-            self.report_waiting.set_visible(False)
-            self.listening_model.clear()
-            self.report_box.hide()
-        self._show_tutorial(self.rules_expander.get_expanded(), not self.report_expander.get_expanded(), self.log_expander.get_expanded())
-    
-    def _show_tutorial(self, rules, report, log):
-        if not rules and not report and not log:
-            self.web.show()
-            self._load_tutorial()
-        else:
-            self.web.hide()
+        self.web_content.load_html_string(html, "file:///")
     
     def _show_web(self, url):
         distro = platform.linux_distribution()[0].lower()
@@ -337,17 +354,6 @@ class Gufw:
         cmd = "su -c 'python -m webbrowser -t \"" + url + "\"' - " + user
         subprocess.Popen(cmd, shell=True)
     
-    def on_expanderLog_activate(self, widget, data=None):
-        if not self.log_expander.get_expanded():
-            self.log_box.show()
-            self.log_txt.set_text('')
-            self.add_to_log(self.fw.get_log(), self.GRAY, False)
-            self.fw.set_config_value('ExpandLog', 'yes')
-        else:
-            self.log_box.hide()
-            self.fw.set_config_value('ExpandLog', 'no')
-        self._show_tutorial(self.rules_expander.get_expanded(), self.report_expander.get_expanded(), not self.log_expander.get_expanded())
-    
     def on_menu_import_activate(self, widget, data=None):
         import_profile = self._file_dialog('open', _("Import Profile"))
         profile = os.path.basename(import_profile) #Filename
@@ -363,7 +369,7 @@ class Gufw:
         
         if profile in self.fw.get_all_profiles():
             self.set_statusbar_msg(_("Operation cancelled"))
-            self.show_dialog(self.winMain, _("Error"), _("Profile already exists"))
+            self.show_dialog(self.winMain, _("Profile already exists"), _("Delete it before from the Preferences Window or rename the file (the profile will be the filename)"))
         else:
             self.fw.import_profile(import_profile)
             self.profile.append_text(profile)
@@ -397,9 +403,12 @@ class Gufw:
     
     def _set_shield(self):
         if self.fw.get_status():
-            file_shield = os.path.join('/usr/share/gufw/media/shields/', self.fw.get_policy('incoming').lower() + '_' + self.fw.get_policy('outgoing').lower() + '.png')
+            file_shield = os.path.join('/usr/share/gufw/media/shields/', self.fw.get_policy('incoming').lower() + '_' + self.fw.get_policy('outgoing').lower() + '_' + self.fw.get_policy('routed').lower() + '.png')
         else:
-            file_shield = os.path.join('/usr/share/gufw/media/shields/', 'disabled_disabled.png')
+            if self.fw.get_policy('routed') == 'disabled':
+                file_shield = os.path.join('/usr/share/gufw/media/shields/', 'disabled_disabled_disabled.png')
+            else:
+                file_shield = os.path.join('/usr/share/gufw/media/shields/', 'disabled_disabled_enabled.png')
         self.shield.set_from_file(file_shield)
     
     def on_menu_reset_activate(self, widget, data=None):
@@ -477,6 +486,17 @@ class Gufw:
         if self.fw.get_policy('outgoing') != self.NUM2POLICY[self.outgoing.get_active()].lower():
             self.show_dialog(self.winMain, _("There was an error changing the outgoing policy"), _("Restart your firewall to refresh to the real status\nand please report this bug"))
     
+    def on_routed_changed(self, widget, data=None):
+        try: # Could be inconsistent between imports and changes of Routed status
+            self.fw.set_policy('routed', self.NUM2POLICY[self.routed.get_active()].lower())
+        except:
+            return
+        self._set_shield()
+        self.add_to_log(_("Routed: ") + self.routed.get_active_text(), self.POLICY2COLOR[self.fw.get_policy('routed')])
+        self.set_statusbar_msg(_("Routed policy changed"))
+        if self.fw.get_policy('routed') != self.NUM2POLICY[self.routed.get_active()].lower():
+            self.show_dialog(self.winMain, _("There was an error changing the routed policy"), _("Restart your firewall to refresh to the real status\nand please report this bug"))
+    
     def on_btnAddRule_clicked(self, widget, data=None):
         self.show_add_btn.set_sensitive(False)
         self.edit_rule_btn.set_sensitive(False)
@@ -518,6 +538,7 @@ class Gufw:
         
         self.incoming.set_sensitive(self.fw.get_status())
         self.outgoing.set_sensitive(self.fw.get_status())
+        self.routed.set_sensitive(self.fw.get_status())
         self.show_add_btn.set_sensitive(self.fw.get_status())
         self.detele_rule_btn.set_sensitive(self.fw.get_status())
         self.edit_rule_btn.set_sensitive(self.fw.get_status())
@@ -534,7 +555,7 @@ class Gufw:
         # Compose real numbers
         for row in rows:
             iter_row = self.rules_model.get_iter(row,)
-            total_rules.append(self.rules_model.get_value(iter_row, 13))
+            total_rules.append(self.rules_model.get_value(iter_row, 14))
         total_rules = sorted(total_rules, key=int, reverse=True)
         
         # Confirm?
@@ -594,10 +615,11 @@ class Gufw:
         to_ip       = self.rules_model.get_value(iter_row, 8)
         to_port     = self.rules_model.get_value(iter_row, 9)
         iface       = self.rules_model.get_value(iter_row, 10)
-        logging     = self.rules_model.get_value(iter_row, 11)
-        ufw_row     = self.rules_model.get_value(iter_row, 13)
+        routed      = self.rules_model.get_value(iter_row, 11)
+        logging     = self.rules_model.get_value(iter_row, 12)
+        ufw_row     = self.rules_model.get_value(iter_row, 14)
         
-        updatewin = Update(self, ufw_row, description, cmd, policy, direction, proto, from_ip, from_port, to_ip, to_port, iface, logging)
+        updatewin = Update(self, ufw_row, description, cmd, policy, direction, proto, from_ip, from_port, to_ip, to_port, iface, routed, logging)
 
     def on_profile_changed(self, widget, data=None):
         operation = self.fw.set_profile(self.profile.get_active_text())
@@ -605,6 +627,7 @@ class Gufw:
         
         self.incoming.set_active(self.POLICY2NUM[self.fw.get_policy('incoming')])
         self.outgoing.set_active(self.POLICY2NUM[self.fw.get_policy('outgoing')])
+        self.routed.set_active(self.POLICY2NUM[self.fw.get_policy('routed')])
         self.switchStatus.set_active(self.fw.get_status())
         
         self.print_rules(self.fw.get_rules())
@@ -669,15 +692,18 @@ class Gufw:
         row = 1
         for rule in (rules):
             iter_row = self.rules_model.insert(row)
-            translated_rule = rule['ufw_rule'].replace(" ALLOW ", _(" ALLOW "))
-            translated_rule = translated_rule.replace(" DENY ",   _(" DENY "))
-            translated_rule = translated_rule.replace(" REJECT ", _(" REJECT "))
-            translated_rule = translated_rule.replace(" LIMIT ",  _(" LIMIT "))
-            translated_rule = translated_rule.replace(" OUT ",    _(" OUT "))
-            translated_rule = translated_rule.replace(" IN ",     _(" IN "))
-            translated_rule = translated_rule.replace("Anywhere", _("Anywhere"))
-            translated_rule = translated_rule.replace(" (out)",   _(" (out)"))
-            translated_rule = translated_rule.replace(" on ",     _(" on "))
+            translated_rule = rule['ufw_rule'].replace(" ALLOW ",  _(" ALLOW "))
+            translated_rule = translated_rule.replace(" DENY ",    _(" DENY "))
+            translated_rule = translated_rule.replace(" REJECT ",  _(" REJECT "))
+            translated_rule = translated_rule.replace(" LIMIT ",   _(" LIMIT "))
+            translated_rule = translated_rule.replace(" OUT ",     _(" OUT "))
+            translated_rule = translated_rule.replace(" IN ",      _(" IN "))
+            translated_rule = translated_rule.replace(" FWD ",     _(" FWD "))
+            translated_rule = translated_rule.replace("Anywhere",  _("Anywhere"))
+            translated_rule = translated_rule.replace(" (log) ",     _("(log)"))
+            translated_rule = translated_rule.replace(" (log-all) ", _("(log-all)"))
+            translated_rule = translated_rule.replace(" (out)",    _(" (out)"))
+            translated_rule = translated_rule.replace(" on ",      _(" on "))
             
             self.rules_model.set_value(iter_row, 0,  translated_rule)     # ufw rule
             self.rules_model.set_value(iter_row, 1,  rule['description']) # description
@@ -690,19 +716,20 @@ class Gufw:
             self.rules_model.set_value(iter_row, 8,  rule['to_ip'])       # to_ip
             self.rules_model.set_value(iter_row, 9,  rule['to_port'])     # to_port
             self.rules_model.set_value(iter_row, 10, rule['iface'])       # iface
-            self.rules_model.set_value(iter_row, 11, rule['logging'])     # logging
-            self.rules_model.set_value(iter_row, 13, row)                 # number
+            self.rules_model.set_value(iter_row, 11, rule['routed'])      # routed
+            self.rules_model.set_value(iter_row, 12, rule['logging'])     # logging
+            self.rules_model.set_value(iter_row, 14, row)                 # number
             
             if 'ALLOW' in rule['ufw_rule']:
-                self.rules_model.set_value(iter_row, 12, self.POLICY2COLOR['allow'])  # color
+                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['allow'])  # color
             elif 'DENY' in rule['ufw_rule']:
-                self.rules_model.set_value(iter_row, 12, self.POLICY2COLOR['deny'])   # color
+                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['deny'])   # color
             elif 'REJECT' in rule['ufw_rule']:
-                self.rules_model.set_value(iter_row, 12, self.POLICY2COLOR['reject']) # color
+                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['reject']) # color
             elif 'LIMIT' in rule['ufw_rule']:
-                self.rules_model.set_value(iter_row, 12, self.POLICY2COLOR['limit'])  # color
+                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['limit'])  # color
             else:
-                self.rules_model.set_value(iter_row, 12, self.POLICY2COLOR['others']) # color
+                self.rules_model.set_value(iter_row, 13, self.POLICY2COLOR['others']) # color
             
             row += 1
     
@@ -738,10 +765,10 @@ class Gufw:
     
     def _restore_window_size(self, win):
         # Fix GUI for small screens:
-        if Gdk.Screen.height() <= 650:
-            win.set_size_request(360, 530)
+        if Gdk.Screen.height() <= 600:
+            win.set_size_request(365, 480)
         if Gdk.Screen.height() <= 480:
-            win.set_size_request(360, 410)
+            win.set_size_request(365, 390)
         
         width = 0
         height = 0
@@ -760,15 +787,19 @@ class Gufw:
         self.fw.set_config_value('WindowWidth',  str(width))
         self.fw.set_config_value('WindowHeight', str(height))
     
-    def validate_rule(self, win, from_ip, from_port, to_ip, to_port):
+    def validate_rule(self, win, from_ip, from_port, to_ip, to_port, insert='', routed=''):
         # At least 1 Port/IP
-        if not from_ip and not from_port and not to_ip and not to_port:
+        if not from_ip and not from_port and not to_ip and not to_port and routed == "Not Forward":
             self.show_dialog(win, _("Insert IP/Ports"), _("You need to insert IP/ports in to/from fields"))
             return False
         
-        # If 2 ports > 2 IPs
-        if (from_port and to_port) and (not from_ip or not to_ip):
-            self.show_dialog(win, _("Insert both IPs"), _("You need to insert both IPs if you're using both ports"))
+        # Not allow insert number bigger that number of rules
+        if not insert:
+            insert_into = 0
+        else:
+            insert_into = int(insert)
+        if insert_into > 0 and insert_into > self.fw.get_number_rules():
+            self.show_dialog(win, _("Insert number bigger that number of rules"), _("By example, if you have 3 rules, you can't insert a rule into position 4"))
             return False
-            
+        
         return True
