@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Gufw 14.04.2 - http://gufw.org
+# Gufw 14.10.1 - http://gufw.org
 # Copyright (C) 2008-2014 Marcos Alvarez Costales https://launchpad.net/~costales
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -25,15 +25,21 @@ class ListeningReport():
     
     def __init__(self, gufw):
         self.gufw = gufw
-        
         self.previous_report = []
-        self.running_listening = False
-    
+        self.running_listening = True
+        
+        self._show_report()
+        
+        if self.gufw.fw.get_config_value('RefreshInterval'):
+            time = int(self.gufw.fw.get_config_value('RefreshInterval'))
+        else:
+            time = REFRESH_TIME
+        GLib.timeout_add((time * 1000), self._show_report)
+        
     def _show_report(self):
         if not self.running_listening:
             return False
         
-        self.gufw.report_waiting.set_visible(False)
         report = self.gufw.fw.get_listening_report()
         self._view_report(report, self.previous_report)
         self.previous_report = report
@@ -43,17 +49,6 @@ class ListeningReport():
         
         return True
     
-    def starting(self):
-        if self.running_listening:
-            return
-        
-        self.running_listening = True
-        if self.gufw.fw.get_config_value('RefreshInterval'):
-            time = int(self.gufw.fw.get_config_value('RefreshInterval'))
-        else:
-            time = REFRESH_TIME
-        GLib.timeout_add((time * 1000), self._show_report)
-    
     def stopping(self):
         self.running_listening = False
     
